@@ -13,7 +13,7 @@ src="$2"
 tempdir=${3:-$(mktemp -d)}
 sfx=${4:-/dists}
 # Debian is easy, Ubuntu is bad... no symbolic name. Try to match well known release names
-testkey=${5:-'updates\|stable\|unstable\|security\|backports\|feisty\|gutsy\|hardy\|intrepid\|jaunty\|karmic\|lucid\|maverick\|depper\|natty\|oneiric\|dapper\|edgy\|precise\|quantal\|raring\|saucy\|trusty'}
+testkey=${5:-'updates\|stable\|unstable\|security\|backports\|feisty\|gutsy\|hardy\|intrepid\|jaunty\|karmic\|lucid\|maverick\|depper\|natty\|oneiric\|dapper\|edgy\|precise\|quantal\|raring\|saucy\|trusty\|vivid\|wily'}
 
 mkdir -p $tempdir
 rm -f $tempdir/url.* $tempdir/log.*
