@@ -23,16 +23,16 @@ typedef std::list<tDlJobPtr> tDljQueue;
 class dlcon : public lockable
 { 
     public:
-        dlcon(bool bManualExecution, mstring *xff=NULL);
+        dlcon(bool bManualExecution, mstring *xff=nullptr);
         ~dlcon();
 
         void WorkLoop();
         
         void SignalStop();
 
-        bool AddJob(tFileItemPtr m_pItem, const tHttpUrl *pForcedUrl=nullptr,
-        		const acfg::tRepoData *pRepoDesc=nullptr,
-        		cmstring *sPatSuffix=nullptr);
+        bool AddJob(tFileItemPtr m_pItem, const tHttpUrl *pForcedUrl,
+        		const acfg::tRepoData *pRepoDesc,
+        		cmstring *sPatSuffix, LPCSTR reqHead);
 
         mstring m_sXForwardedFor;
 
