@@ -35,17 +35,22 @@ public:
 	//! @return False on errors.
 	bool GetOneLine(mstring & sOut, bool bForceUncompress=false);
 	uint GetCurrentLine() const { return m_nCurLine;} ;
-	bool CheckGoodState(bool bTerminateOnErrors, cmstring *reportFilePath=NULL) const;
+	bool CheckGoodState(bool bTerminateOnErrors, cmstring *reportFilePath=nullptr) const;
 	
-	bool GetChecksum(int csType, uint8_t out[], off_t &scannedSize, FILE *pDumpFile=NULL);
+	bool GetChecksum(int csType, uint8_t out[], off_t &scannedSize, FILE *pDumpFile=nullptr);
 	static bool GetChecksum(const mstring & sFileName, int csType, uint8_t out[],
-			bool bTryUnpack, off_t &scannedSize, FILE *pDumpFile=NULL);
+			bool bTryUnpack, off_t &scannedSize, FILE *pDumpFile=nullptr);
 
 	inline const char *GetBuffer() const { return m_szFileBuf; };
 	inline size_t GetSize() const { return m_nBufSize; };
 
 	void Close();
 
+	const mstring& getSErrorString() const
+	{
+		return m_sErrorString;
+	}
+
 private:
 
 	bool m_bError, m_bEof;
@@ -64,13 +69,12 @@ private:
 	
 	int m_nEofLines;
 
-	std::auto_ptr<IDecompressor> m_Dec;
+	std::unique_ptr<IDecompressor> m_Dec;
 
 	// not to be copied
 	filereader& operator=(const filereader&);
 	filereader(const filereader&);
-
-	std::unique_ptr<filelocks::flock> m_mmapLock;
+	std::unique_ptr<TFileShrinkGuard> m_mmapLock;
 };
 
 extern uint_fast16_t hexmap[];
