@@ -53,7 +53,7 @@ string tmpDontcache, tmpDontcacheReq, tmpDontcacheTgt;
 tStrMap localdirs;
 static class : public lockable, public NoCaseStringMap {} mimemap;
 
-std::bitset<TCP_PORT_MAX> *pUserPorts = NULL;
+std::bitset<TCP_PORT_MAX> *pUserPorts = nullptr;
 
 struct MapNameToString
 {
@@ -163,8 +163,8 @@ MapNameToInt n2iTbl[] = {
 		,{ "NoSSLchecks",	&nsafriendly, 		"Disable SSL security checks" , 10}
 };
 
-void ReadRewriteFile(const string & sFile, const string & sRepName);
-void ReadBackendsFile(const string & sFile, const string &sRepName);
+uint ReadBackendsFile(const string & sFile, const string &sRepName);
+uint ReadRewriteFile(const string & sFile, cmstring& sRepName);
 
 map<cmstring, tRepoData> repoparms;
 typedef decltype(repoparms)::iterator tPairRepoNameData;
@@ -243,10 +243,10 @@ inline void _FixPostPreSlashes(string &val)
 		val.insert(0, "/", 1);
 }
 
-bool ReadOneConfFile(const string & szFilename)
+bool ReadOneConfFile(const string & szFilename, bool bReadErrorIsFatal=true)
 {
 	tCfgIter itor(szFilename);
-	itor.reader.CheckGoodState(true, &szFilename);
+	itor.reader.CheckGoodState(bReadErrorIsFatal, &szFilename);
 
 	NoCaseStringMap dupeCheck;
 
@@ -320,7 +320,7 @@ inline void AddRemapFlag(const string & token, const string &repname)
 		if (acfg::debug&LOG_FLUSH)
 			cerr << "Fatal keyfile for " <<repname<<": "<<value <<endl;
 
-		where.m_keyfiles.push_back(value);
+		where.m_keyfiles.emplace_back(value);
 	}
 	else if(key=="deltasrc")
 	{
@@ -339,7 +339,7 @@ inline void AddRemapFlag(const string & token, const string &repname)
 		tHttpUrl cand;
 		if(value.empty() || cand.SetHttpUrl(value))
 		{
-			alt_proxies.push_back(cand);
+			alt_proxies.emplace_back(cand);
 			where.m_pProxy = & alt_proxies.back();
 		}
 		else
@@ -350,9 +350,8 @@ inline void AddRemapFlag(const string & token, const string &repname)
 	}
 }
 
-tStrDeq ExpandFileTokens(cmstring &token, bool bUseDefaultFallback)
+tStrDeq ExpandFileTokens(cmstring &token)
 {
-	tStrDeq srcs;
 	string sPath = token.substr(5);
 	if (sPath.empty())
 		BARF("Bad file spec for repname, file:?");
@@ -361,64 +360,26 @@ tStrDeq ExpandFileTokens(cmstring &token, bool bUseDefaultFallback)
 	{
 		if (!bAbs)
 			sPath = confdir + sPathSep + sPath;
-		srcs = ExpandFilePattern(sPath, true);
+		return ExpandFilePattern(sPath, true);
 	}
-	else
-	{
-		tStrMap bname2path;
-#ifdef DEBUG
-		bool bp=(token == "file:backends_gentoo");
-#endif
-		auto lookup = [&sPath, &srcs, &bname2path](bool bAddDefault)
-		{
-			for (const auto& dir : { &confdir, &suppdir })
-			{
-				// chop slashes. That should not be required but better be sure.
-				while(endsWithSzAr(*dir, sPathSep) && dir->size()>1)
-				dir->resize(dir->size()-1);
-
-				auto pat=(cmstring) *dir + sPathSep + sPath;
-				if(bAddDefault)
-					pat+=".default";
-				//cerr << "heh, token: " << token << " in dir: " << *dir << " to: " << pat;
-				srcs = ExpandFilePattern(pat, true);
-				if (srcs.size() == 1 && !Cstat(srcs.front()))
-					continue;// file not existing, wildcard returned
-
-				// hits in confdir will overwrite those from suppdir
-				for (auto& s: srcs)
-				{
-					auto nam(GetBaseName(s));
-					if(bAddDefault)
-					nam.erase(nam.size()-8);
-#if __GNUC__ >= 4 && __GNUC_MINOR__ < 8
-					if(bname2path.find(nam) == bname2path.end())
-						bname2path[nam]=s;
-#else
-					bname2path.emplace(nam, s);
-#endif
-				}
-
-			}
-		};
-		lookup(false);
-		if(bUseDefaultFallback && bname2path.empty())
-		{
-			if(debug>4)
-				cerr << "Looking for fallback version: " << sPath << ".default" <<endl;
-			lookup(true);
-		}
-
-		if (bname2path.empty())
-		{
-			cerr << "WARNING: No URL list file matching " << token
-					<< " found in config or support directories." << endl;
-		}
-		srcs.clear();
-		for (auto& b2p : bname2path)
-			srcs.push_back(b2p.second);
-	}
-	return srcs;
+	auto pat = confdir + sPathSep + sPath;
+	StrSubst(pat, "//", "/");
+	auto res = ExpandFilePattern(pat, true);
+	if (res.size() == 1 && !Cstat(res.front()))
+		res.clear(); // not existing, wildcard returned
+	pat = suppdir + sPathSep + sPath;
+	StrSubst(pat, "//", "/");
+	auto suppres = ExpandFilePattern(pat, true);
+	if (suppres.size() == 1 && !Cstat(suppres.front()))
+		return res; // errrr... done here
+	// merge them
+	tStrSet dupeFil;
+	for(const auto& s: res)
+		dupeFil.emplace(GetBaseName(s));
+	for(const auto& s: suppres)
+		if(!ContHas(dupeFil, GetBaseName(s)))
+			res.emplace_back(s);
+	return res;
 }
 
 inline void AddRemapInfo(bool bAsBackend, const string & token,
@@ -432,20 +393,22 @@ inline void AddRemapInfo(bool bAsBackend, const string & token,
 		_FixPostPreSlashes(url.sPath);
 
 		if (bAsBackend)
-			repoparms[repname].m_backends.push_back(url);
+			repoparms[repname].m_backends.emplace_back(url);
 		else
-			mapUrl2pVname[url.sHost+":"+url.GetPort()].push_back(
-					make_pair(url.sPath, GetRepoEntryRef(repname)));
+			mapUrl2pVname[url.sHost+":"+url.GetPort()].emplace_back(
+					url.sPath, GetRepoEntryRef(repname));
 	}
 	else
 	{
-		for(auto& src : ExpandFileTokens(token, true))
-		{
-			if (bAsBackend)
-				ReadBackendsFile(src, repname);
-			else
-				ReadRewriteFile(src, repname);
-		}
+		auto func = bAsBackend ? ReadBackendsFile : ReadRewriteFile;
+		uint count = 0;
+		for(auto& src : ExpandFileTokens(token))
+			count += func(src, repname);
+		if(!count)
+			for(auto& src : ExpandFileTokens(token + ".default"))
+				count = func(src, repname);
+		if(!count && !g_bQuiet)
+			cerr << "WARNING: No configuration was read from " << token << endl;
 	}
 }
 
@@ -510,7 +473,7 @@ void _AddHooksFile(cmstring& vname)
 		else if (strcasecmp("DownTimeout", p) == 0)
 		{
 			errno = 0;
-			uint n = strtoul(val.c_str(), NULL, 10);
+			uint n = strtoul(val.c_str(), nullptr, 10);
 			if (!errno)
 				hs.downDuration = n;
 		}
@@ -679,13 +642,18 @@ bool SetOption(const string &sLine, NoCaseStringMap *pDupeCheck)
 				BARF("Invalid proxy specification, aborting...");
 		}
 	}
-	else if(CHECKOPTKEY("LocalDirs") && !g_bNoComplex)
+	else if(CHECKOPTKEY("LocalDirs"))
 	{
+		if(g_bNoComplex)
+			return true;
 		_ParseLocalDirs(value);
 		return !localdirs.empty();
 	}
-	else if(0==strncasecmp(key.c_str(), "Remap-", 6) && !g_bNoComplex)
+	else if(0==strncasecmp(key.c_str(), "Remap-", 6))
 	{
+		if(g_bNoComplex)
+			return true;
+
 		string vname=key.substr(6, key.npos);
 		if(vname.empty())
 		{
@@ -822,10 +790,9 @@ const tRepoData * GetRepoData(cmstring &vname)
 	return & it->second;
 }
 
-void ReadBackendsFile(const string & sFile, const string &sRepName)
+uint ReadBackendsFile(const string & sFile, const string &sRepName)
 {
-
-	int nAddCount=0;
+	uint nAddCount=0;
 	string key, val;
 	tHttpUrl entry;
 
@@ -837,7 +804,7 @@ void ReadBackendsFile(const string & sFile, const string &sRepName)
 	{
 		if(debug&6)
 			cerr << "No backend data found, file ignored."<<endl;
-		return;
+		return 0;
 	}
 	
 	while(itor.Next())
@@ -854,7 +821,7 @@ void ReadBackendsFile(const string & sFile, const string &sRepName)
 #ifdef DEBUG
 			cerr << "Backend: " << sRepName << " <-- " << entry.ToURI(false) <<endl;
 #endif		
-			repoparms[sRepName].m_backends.push_back(entry);
+			repoparms[sRepName].m_backends.emplace_back(entry);
 			nAddCount++;
 			entry.clear();
 		}
@@ -871,6 +838,7 @@ void ReadBackendsFile(const string & sFile, const string &sRepName)
 					<< itor.reader.GetCurrentLine());
 		}
 	}
+	return nAddCount;
 }
 
 void ShutDown()
@@ -882,8 +850,9 @@ void ShutDown()
 /* This parses also legacy files, i.e. raw RFC-822 formated mirror catalogue from the
  * Debian archive maintenance repository.
  */
-void ReadRewriteFile(const string & sFile, cmstring& sRepName)
+uint ReadRewriteFile(const string & sFile, cmstring& sRepName)
 {
+	uint nAddCount=0;
 	filereader reader;
 	if(debug>4)
 		cerr << "Reading rewrite file: " << sFile <<endl;
@@ -893,24 +862,25 @@ void ReadRewriteFile(const string & sFile, cmstring& sRepName)
 	tStrVec hosts, paths;
 	string sLine, key, val;
 	tHttpUrl url;
-	
-	while(reader.GetOneLine(sLine))
+
+	while (reader.GetOneLine(sLine))
 	{
 		trimFront(sLine);
 
-		if (0==sLine.compare(0, 1, "#"))
+		if (0 == sLine.compare(0, 1, "#"))
 			continue;
 
 		if (url.SetHttpUrl(sLine))
 		{
 			_FixPostPreSlashes(url.sPath);
 
-			mapUrl2pVname[url.sHost+":"+url.GetPort()].push_back(
-					make_pair(url.sPath, GetRepoEntryRef(sRepName)));
+			mapUrl2pVname[url.sHost + ":" + url.GetPort()].emplace_back(url.sPath,
+					GetRepoEntryRef(sRepName));
 #ifdef DEBUG
-						cerr << "Mapping: "<< url.ToURI(false)
-						<< " -> "<< sRepName <<endl;
+			cerr << "Mapping: " << url.ToURI(false) << " -> " << sRepName << endl;
 #endif
+
+			++nAddCount;
 			continue;
 		}
 
@@ -938,13 +908,15 @@ void ReadRewriteFile(const string & sFile, cmstring& sRepName)
 					tHttpUrl url;
 					url.sHost=host;
 					url.sPath=path;
-					mapUrl2pVname[url.sHost+":"+url.GetPort()].push_back(
-						make_pair(url.sPath, GetRepoEntryRef(sRepName)));
+					mapUrl2pVname[url.sHost+":"+url.GetPort()].emplace_back(url.sPath,
+							GetRepoEntryRef(sRepName));
 
 #ifdef DEBUG
 						cerr << "Mapping: "<< host << path
 						<< " -> "<< sRepName <<endl;
 #endif
+
+						++nAddCount;
 				}
 			}
 			hosts.clear();
@@ -966,15 +938,17 @@ void ReadRewriteFile(const string & sFile, cmstring& sRepName)
 		{
 			// help STL saving some memory
 			if(sPopularPath==val)
-				paths.push_back(sPopularPath);
+				paths.emplace_back(sPopularPath);
 			else
 			{
 				_FixPostPreSlashes(val);
-				paths.push_back(val);
+				paths.emplace_back(val);
 			}
 			continue;
 		}
 	}
+
+	return nAddCount;
 }
 
 
@@ -983,7 +957,7 @@ tRepoData::~tRepoData()
 	delete m_pHooks;
 }
 
-void ReadConfigDirectory(const char *szPath)
+void ReadConfigDirectory(const char *szPath, bool bReadErrorIsFatal)
 {
 	dump_proc_status();
 	char buf[PATH_MAX];
@@ -994,9 +968,9 @@ void ReadConfigDirectory(const char *szPath)
 
 #if defined(HAVE_WORDEXP) || defined(HAVE_GLOB)
 	for(const auto& src: ExpandFilePattern(confdir+SZPATHSEP "*.conf", true))
-		ReadOneConfFile(src);
+		ReadOneConfFile(src, bReadErrorIsFatal);
 #else
-	ReadOneConfFile(confdir+SZPATHSEP"acng.conf", bQuiet, bNoComplex);
+	ReadOneConfFile(confdir+SZPATHSEP"acng.conf", bReadErrorIsFatal);
 #endif
 	dump_proc_status();
 	if(debug & LOG_DEBUG)
@@ -1012,6 +986,7 @@ void ReadConfigDirectory(const char *szPath)
 
 void PostProcConfig()
 {
+	mapUrl2pVname.rehash(mapUrl2pVname.size());
 	
 	if(port.empty()) // heh?
 		port=ACNG_DEF_PORT;
@@ -1084,7 +1059,7 @@ void PostProcConfig()
 	   mkbasedir(acfg::pidfile);
 
    if(nettimeout < 5) {
-	   cerr << "Warning, NetworkTimeout too small, assuming 5." << endl;
+	   cerr << "Warning, NetworkTimeout value too small, using: 5." << endl;
 	   nettimeout = 5;
    }
 
@@ -1368,9 +1343,9 @@ bool CompileExpressions()
 
 inline bool MatchType(cmstring &in, eMatchType type)
 {
-	if(rex[type].pat && !regexec(rex[type].pat, in.c_str(), 0, NULL, 0))
+	if(rex[type].pat && !regexec(rex[type].pat, in.c_str(), 0, nullptr, 0))
 		return true;
-	if(rex[type].extra && !regexec(rex[type].extra, in.c_str(), 0, NULL, 0))
+	if(rex[type].extra && !regexec(rex[type].extra, in.c_str(), 0, nullptr, 0))
 		return true;
 	return false;
 }
@@ -1411,7 +1386,7 @@ inline bool CompileUncachedRex(const string & token, NOCACHE_PATTYPE type, bool
 	}
 	else if(!bRecursiveCall) // don't go further than one level
 	{
-		tStrDeq srcs = acfg::ExpandFileTokens(token, true);
+		tStrDeq srcs = acfg::ExpandFileTokens(token);
 		for(const auto& src: srcs)
 		{
 			acfg::tCfgIter itor(src);
@@ -1446,7 +1421,7 @@ bool CompileUncExpressions(NOCACHE_PATTYPE type, cmstring& pat)
 bool MatchUncacheable(const string & in, NOCACHE_PATTYPE type)
 {
 	for(const auto& patre: (type == NOCACHE_REQ) ? vecReqPatters : vecTgtPatterns)
-		if(!regexec(&patre, in.c_str(), 0, NULL, 0))
+		if(!regexec(&patre, in.c_str(), 0, nullptr, 0))
 			return true;
 	return false;
 }
