@@ -51,6 +51,7 @@ string vfilepat(INFOLDER
 		")$" // end of only-filename paterns
 		"|/dists/.*/installer-[^/]+/[^0-9][^/]+/images/.*" // d-i stuff but not containing a date (year number) in the revision directory (like "current", "beta", ...)
 		"|/pks/lookup.op.get" // some Ubuntu PPA management activity
+		"|centos/.*/images/.*img" // [#314924] Allow access to CentOS images
 );
 
 //string wfilepat( VPATPREFIX  "(Release|Release\\.gpg|release|meta-release|Translation[^/]*\\.bz2)$");
@@ -60,6 +61,7 @@ string wfilepat(INFOLDER
 		"|(Packages|Sources)" COMPRLIST "?" // hm... private repos without Release file :-(
 		"|Translation[^/]*" COMPRLIST "?" // to be checked, but they should never really go anywhere
 		"|.*\\.xml" // SUSE
+		"|setup\\.bz2(.sig)?" // Cygwin
 		"|" ALXPATTERN // Arch Linux
 		"|[a-z]+32.exe"
 		")$");
@@ -72,7 +74,7 @@ keepnver(0), maxtempdelay(27), vrangeops(1);
 int dlbufsize(70000), exfailabort(1), exporigin(false), numcores(1),
 logxff(false), oldupdate(false), recompbz2(false), nettimeout(60), updinterval(0),
 forwardsoap(RESERVED_DEFVAL), usewrap(RESERVED_DEFVAL), redirmax(RESERVED_DEFVAL),
-stucksecs(500), persistoutgoing(1), pipelinelen(255), exsupcount(RESERVED_DEFVAL),
+stucksecs(500), persistoutgoing(1), pipelinelen(10), exsupcount(RESERVED_DEFVAL),
 optproxytimeout(-1), patrace(false), maxredlsize(1<<16), nsafriendly(false);
 
 int maxdlspeed(RESERVED_DEFVAL);
