@@ -32,6 +32,8 @@
 //#define DEBUGSPAM
 #endif
 
+#define MAX_TOP_COUNT 10
+
 using namespace std;
 
 static cmstring oldStylei18nIdx("/i18n/Index");
@@ -49,7 +51,7 @@ tCacheOperation::tCacheOperation(const tSpecialRequest::tRunParms& parms) :
 	m_bScanInternals(false), m_bByPath(false), m_bByChecksum(false), m_bSkipHeaderChecks(false),
 	m_bTruncateDamaged(false),
 	m_nErrorCount(0),
-	m_nProgIdx(0), m_nProgTell(1), m_pDlcon(NULL)
+	m_nProgIdx(0), m_nProgTell(1), m_pDlcon(nullptr)
 {
 	m_szDecoFile="maint.html";
 	m_gMaintTimeNow=GetTime();
@@ -59,7 +61,7 @@ tCacheOperation::tCacheOperation(const tSpecialRequest::tRunParms& parms) :
 tCacheOperation::~tCacheOperation()
 {
 	delete m_pDlcon;
-	m_pDlcon=NULL;
+	m_pDlcon=nullptr;
 }
 
 bool tCacheOperation::ProcessOthers(const string &, const struct stat &)
@@ -260,7 +262,7 @@ bool tCacheOperation::Download(cmstring& sFilePathRel, bool bIsVolatileFile,
 				&& !pRepoDesc->m_backends.empty())
 		{
 			ldbg("will use backend mode, subdirectory is path suffix relative to backend uri");
-			sRemoteSuffix=parserPath.sPath;
+			sRemoteSuffix=parserPath.sPath.substr(1);
 		}
 		else
 		{
@@ -270,11 +272,11 @@ bool tCacheOperation::Download(cmstring& sFilePathRel, bool bIsVolatileFile,
 			// the URL as needed
 
 			if(bCachePathAsUriPlausible) // unless other rule matches, this path in cache should represent the remote URI
-				pResolvedDirectUrl=&parserPath;
+				pResolvedDirectUrl = parserPath.NormalizePath();
 
 			// and prefer the source from xorig which is likely to deliver better result
 			if(hor.h[header::XORIG] && parserHead.SetHttpUrl(hor.h[header::XORIG], false))
-				pResolvedDirectUrl=&parserHead;
+				pResolvedDirectUrl = parserHead.NormalizePath();
 			else if(flags.guessed)
 			{
 				// might use a related file as reference
@@ -296,7 +298,7 @@ bool tCacheOperation::Download(cmstring& sFilePathRel, bool bIsVolatileFile,
 							xpath.erase(xpath.size() - diffIdxSfx.size());
 							xpath += sFilePathAbs.substr(pos);
 							if (parserHead.SetHttpUrl(xpath, false))
-								pResolvedDirectUrl = &parserHead;
+								pResolvedDirectUrl = parserHead.NormalizePath();
 						}
 					}
 				}
@@ -339,7 +341,7 @@ bool tCacheOperation::Download(cmstring& sFilePathRel, bool bIsVolatileFile,
 								urlcand += *pCompSuf;
 								if (parserHead.SetHttpUrl(urlcand, false))
 								{
-									pResolvedDirectUrl = &parserHead;
+									pResolvedDirectUrl = parserHead.NormalizePath();
 									break;
 								}
 							}
@@ -370,10 +372,10 @@ bool tCacheOperation::Download(cmstring& sFilePathRel, bool bIsVolatileFile,
 		}
 	}
 
-	m_pDlcon->AddJob(pFi, pResolvedDirectUrl, pRepoDesc, &sRemoteSuffix);
+	m_pDlcon->AddJob(pFi, pResolvedDirectUrl, pRepoDesc, &sRemoteSuffix, 0);
 
 	m_pDlcon->WorkLoop();
-	if (pFi->WaitForFinish(NULL) == fileitem::FIST_COMPLETE
+	if (pFi->WaitForFinish(nullptr) == fileitem::FIST_COMPLETE
 			&& pFi->GetHeaderUnlocked().getStatus() == 200)
 	{
 		bSuccess = true;
@@ -487,8 +489,13 @@ bool tCacheOperation::Download(cmstring& sFilePathRel, bool bIsVolatileFile,
 					{
 						newurl.sPath.erase(newurl.sPath.size() - 4);
 						newurl.sPath.append(".xz");
-						return Download(sFilePathRel.substr(0, sFilePathRel.size() - 4) + ".xz",
-								bIsVolatileFile, msgVerbosityLevel, tFileItemPtr(), &newurl);
+						if(Download(sFilePathRel.substr(0, sFilePathRel.size() - 4) + ".xz",
+								bIsVolatileFile, msgVerbosityLevel, tFileItemPtr(), &newurl))
+						{
+							MarkObsolete(sFilePathRel);
+							return true;
+						}
+						return false;
 					}
 				}
 			}
@@ -652,15 +659,16 @@ tFingerprint * BuildPatchList(string sFilePathAbs, deque<tPatchEntry> &retList)
 			else if(tmp[0] == "SHA1-Patches:")
 				eSection=ePatches;
 			else
-				return NULL;
+				return nullptr;
 		}
 		else if(nTokens) // not null but weird count
-			return NULL; // error
+			return nullptr; // error
 	}
 
 	return ret.csType != CSTYPE_INVALID ? &ret : nullptr;
 }
 
+
 bool tCacheOperation::PatchFile(const string &srcRel,
 		const string &diffIdxPathRel, tPListConstIt pit, tPListConstIt itEnd,
 		const tFingerprint *verifData)
@@ -692,7 +700,7 @@ bool tCacheOperation::PatchFile(const string &srcRel,
 
 		SetFlags(pfile).parseignore=true; // not an ifile, never parse this
 		::mkbasedir(sFinalPatch);
-		if(!pf.p && ! (pf.p=fopen(sFinalPatch.c_str(), "w")))
+		if(!pf.p && ! (pf.p=fopen(sFinalPatch.c_str(), "w+")))
 		{
 			SendChunk("Failed to create intermediate patch file, stop patching...<br>");
 			return false;
@@ -730,7 +738,13 @@ bool tCacheOperation::PatchFile(const string &srcRel,
 		SendChunk("Patching...<br>");
 
 	tSS cmd;
-	cmd << "cd '" << CACHE_BASE << "_actmp' && red --silent patch.base < combined.diff";
+	cmd << "cd '" << CACHE_BASE << "_actmp' && ";
+	auto act = acfg::suppdir + SZPATHSEP "acngtool";
+	if(!acfg::suppdir.empty() && 0==access(act.c_str(), X_OK))
+		cmd << "'" << act << "' patch patch.base " << sFinalPatch << " patch.result";
+	else
+		cmd << " red --silent patch.base < " << sFinalPatch;
+
 	if (::system(cmd.c_str()))
 	{
 		MTLOGASSERT(false, "Command failed: " << cmd);
@@ -801,6 +815,12 @@ bool tCacheOperation::Inject(cmstring &from, cmstring &to,
 	if(GetFlags(to).uptodate)
 		return true;
 
+	auto sAbsFrom(SABSPATH(from)), sAbsTo(SABSPATH(to));
+
+	Cstat infoFrom(sAbsFrom), infoTo(sAbsTo);
+	if(infoFrom && infoTo && infoFrom.st_ino == infoTo.st_ino && infoFrom.st_dev == infoTo.st_dev)
+		return true;
+
 #ifdef DEBUG_FLAGS
 	bool nix = stmiss!=from.find("debrep/dists/squeeze/non-free/binary-amd64/");
 	SendFmt<<"Replacing "<<to<<" with " << from <<  "<br>\n";
@@ -812,7 +832,7 @@ bool tCacheOperation::Inject(cmstring &from, cmstring &to,
 	{
 		pHead = &head;
 
-		if (head.LoadFromFile(SABSPATH(from+".head")) <= 0 || !head.h[header::CONTENT_LENGTH])
+		if (head.LoadFromFile(sAbsFrom+".head") <= 0 || !head.h[header::CONTENT_LENGTH])
 		{
 			MTLOGASSERT(0, "Cannot read " << from << ".head or bad data");
 			return false;
@@ -832,7 +852,7 @@ bool tCacheOperation::Inject(cmstring &from, cmstring &to,
 		{
 		}
 		// noone else should attempt to store file through it
-		virtual bool DownloadStartedStoreHeader(const header &, const char *,
+		virtual bool DownloadStartedStoreHeader(const header &, size_t, const char *,
 				bool, bool&) override
 		{
 			return false;
@@ -873,9 +893,10 @@ bool tCacheOperation::Inject(cmstring &from, cmstring &to,
 			if (Setup(true) > fileitem::FIST_COMPLETE)
 				return false;
 			bool bNix(false);
-			if (!fileitem_with_storage::DownloadStartedStoreHeader(head, NULL, false, bNix))
+			if (!fileitem_with_storage::DownloadStartedStoreHeader(head, 0,
+					nullptr, false, bNix))
 				return false;
-			if(!StoreFileData(data.GetBuffer(), data.GetSize()) || ! StoreFileData(NULL, 0))
+			if(!StoreFileData(data.GetBuffer(), data.GetSize()) || ! StoreFileData(nullptr, 0))
 				return false;
 			if(GetStatus() != FIST_COMPLETE)
 				return false;
@@ -1225,7 +1246,7 @@ has_base:
 					sCandId=it2->second;
 			}
 			tClassDesc &tgt=eqClasses[sCandId];
-			tgt.paths.push_back(if2cid.first);
+			tgt.paths.emplace_back(if2cid.first);
 
 			// pick up the id for bz2 verification later
 			if(tgt.bz2VersContId.second.empty() && endsWithSzAr(if2cid.first, ".bz2"))
@@ -1324,7 +1345,7 @@ strip_next_class:
 		int nProbeCnt(3);
 		string patchidxFileToUse;
 		deque<tPatchEntry> patchList;
-		tFingerprint *pEndSum(NULL);
+		tFingerprint *pEndSum(nullptr);
 		tPListConstIt itPatchStart;
 
 		if(CheckStopSignal())
@@ -1395,7 +1416,7 @@ strip_next_class:
 				MTLOGDEBUG("#### Testing file: " << pathRel << " as patch base candidate");
 				if (probe.ScanFile(absPath, CSTYPE_SHA1, true, df.p))
 				{
-					df.close(); // write the whole file to disk ASAP!
+					fflush(df.p); // write the whole file to disk ASAP!
 
 					if(CheckStopSignal())
 						return;
@@ -1444,7 +1465,7 @@ strip_next_class:
 							SendFmt << "Found patching base candidate, unpacked to "
 								<< sPatchBaseAbs << "<br>";
 
-						if (PatchFile(sPatchBaseAbs, patchidxFileToUse, itPatchStart,
+						if (df.close(), PatchFile(sPatchBaseAbs, patchidxFileToUse, itPatchStart,
 									patchList.end(), pEndSum))
 						{
 
@@ -1648,6 +1669,9 @@ tCacheOperation::enumMetaType tCacheOperation::GuessMetaTypeFromURL(cmstring &sP
 	if (sPureIfileName == "MD5SUMS" && StrHas(sPath, "/installer-"))
 		return EIDX_MD5DILIST;
 
+	if (sPureIfileName == "SHA256SUMS" && StrHas(sPath, "/installer-"))
+		return EIDX_SHA256DILIST;
+
 	return EIDX_UNSUPPORTED;
 }
 
@@ -1676,9 +1700,13 @@ bool tCacheOperation::ParseAndProcessMetaFile(ifileprocessor &ret, const std::st
 		// directory starting in dists/?
 		// The assumption doesn't however apply to the d-i checksum
 		// lists, those refer to themselves only.
+		//
+		// similar considerations for Cygwin setup
 
 		if(idxType != EIDX_MD5DILIST && stmiss!=(pos=sCurFilesReferenceDirRel.rfind("/dists/")))
-			sPkgBaseDir.assign(sCurFilesReferenceDirRel, 0, pos+1);
+			sPkgBaseDir = sPkgBaseDir.assign(sCurFilesReferenceDirRel, 0, pos+1);
+		else if(idxType == EIDX_CYGSETUP && stmiss!=(pos=sCurFilesReferenceDirRel.rfind("/cygwin/")))
+			sPkgBaseDir = sPkgBaseDir.assign(sCurFilesReferenceDirRel, 0, pos+8);
 		else
 			sPkgBaseDir=sCurFilesReferenceDirRel;
 	}
@@ -1825,32 +1853,21 @@ bool tCacheOperation::ParseAndProcessMetaFile(ifileprocessor &ret, const std::st
 			if(CheckStopSignal())
 				return true;
 
-			static const string cygkeys[]={"install: ", "source: "};
-
-			trimBack(sLine);
-			for(auto& ckey: cygkeys)
+			uint begin(0);
+			if(startsWithSz(sLine, "install: "))
+				begin=9;
+			else if(startsWithSz(sLine, "source: "))
+				begin=8;
+			else
+				continue;
+			tSplitWalk split(&sLine, SPACECHARS, begin);
+			if(split.Next() && info.SetFromPath(split, sPkgBaseDir)
+					&& split.Next() && info.SetSize(split.remainder())
+					&& split.Next() && info.fpr.SetCs(split))
 			{
-				if(!startsWith(sLine, ckey))
-					continue;
-				if (3 == Tokenize(sLine, "\t ", vsMetrics, false, ckey.length())
-						&& info.fpr.SetCs(vsMetrics[2], CSTYPE_MD5)
-						&& 0>= (info.fpr.size=atoofft(vsMetrics[1].c_str(), -2)))
-				{
-					tStrPos pos = vsMetrics[0].rfind(SZPATHSEPUNIX);
-					if (pos == stmiss)
-					{
-						info.sFileName = vsMetrics[0];
-						info.sDirectory = sCurFilesReferenceDirRel;
-					}
-					else
-					{
-						info.sFileName = vsMetrics[0].substr(pos + 1);
-						info.sDirectory = sCurFilesReferenceDirRel + vsMetrics[0].substr(0, pos + 1);
-					}
-					ret.HandlePkgEntry(info);
-					info.SetInvalid();
-				}
+				ret.HandlePkgEntry(info);
 			}
+			info.SetInvalid();
 		}
 		break;
 	case EIDX_SUSEREPO:
@@ -1898,8 +1915,10 @@ bool tCacheOperation::ParseAndProcessMetaFile(ifileprocessor &ret, const std::st
 			}
 		}
 		break;
+		// like http://ftp.uni-kl.de/debian/dists/jessie/main/installer-amd64/current/images/SHA256SUMS
 	case EIDX_MD5DILIST:
-		LOG("Plain list of filenames and md5sums");
+	case EIDX_SHA256DILIST:
+		LOG("Plain list of filenames and checksums");
 		while(reader.GetOneLine(sLine))
 		{
 			if(CheckStopSignal())
@@ -1907,7 +1926,8 @@ bool tCacheOperation::ParseAndProcessMetaFile(ifileprocessor &ret, const std::st
 
 			tSplitWalk split(&sLine, SPACECHARS);
 			info.fpr.size=-1;
-			if( split.Next() && info.fpr.SetCs(split, CSTYPE_MD5)
+			if( split.Next() && info.fpr.SetCs(split,
+					idxType == EIDX_MD5DILIST ? CSTYPE_MD5 : CSTYPE_SHA256)
 					&& split.Next() && (info.sFileName = split).size()>0)
 			{
 				info.sDirectory = sCurFilesReferenceDirRel;
@@ -1944,8 +1964,8 @@ bool tCacheOperation::ParseAndProcessMetaFile(ifileprocessor &ret, const std::st
 	case EIDX_RELEASE:
 		info.fpr.csType = CSTYPE_SHA1;
 		sStartMark="SHA1:";
-		// fall-through, parser follows
-
+		// parser follows
+		//no break
 	case EIDX_RFC822WITHLISTS:
 		// common info object does not help here because there are many entries, and directory
 		// could appear after the list :-(
@@ -1973,7 +1993,7 @@ bool tCacheOperation::ParseAndProcessMetaFile(ifileprocessor &ret, const std::st
 					sDirHeader=sLine.substr(pos)+SZPATHSEP;
 			}
 			else if (bUse)
-				fileList.push_back(sLine);
+				fileList.emplace_back(sLine);
 			else if(sLine.empty()) // ok, time to commit the list
 			{
 				for (auto& fline : fileList)
@@ -2099,8 +2119,8 @@ void tCacheOperation::ProcessSeenMetaFiles(ifileprocessor &pkgHandler)
 
 void tCacheOperation::AddDelCbox(cmstring &sFileRel)
 {
-	std::pair<tStrSet::iterator,bool> ck = m_delCboxFilter.insert(sFileRel);
-	if(! ck.second)
+	auto ref_isnew = m_delCboxFilter.insert(sFileRel);
+	if(! ref_isnew.second)
 		return;
 
 	SendFmtRemote <<  "<label><input type=\"checkbox\" name=\"kf" << (nKillLfd++)
@@ -2125,18 +2145,17 @@ void tCacheOperation::SetCommonUserFlags(cmstring &cmd)
 	m_bTruncateDamaged=(cmd.find("truncNow")!=stmiss);
 }
 
-
 void tCacheOperation::PrintStats(cmstring &title)
 {
 	multimap<off_t, cmstring*> sorted;
 	off_t total=0;
-	const uint nMax = m_bVerbose ? (UINT_MAX-1) : 10;
+	const uint nMax = m_bVerbose ? (UINT_MAX-1) : MAX_TOP_COUNT;
 	uint hidden=0;
 	for(auto &f: m_metaFilesRel)
 	{
 		total += f.second.space;
 		if(f.second.space)
-			sorted.insert(make_pair(f.second.space, &f.first));
+			EMPLACE_PAIR(sorted,f.second.space, &f.first);
 		if(sorted.size()>nMax)
 		{
 			sorted.erase(sorted.begin());
@@ -2146,7 +2165,7 @@ void tCacheOperation::PrintStats(cmstring &title)
 	if(!total)
 		return;
 	m_fmtHelper << "<br>\n<table><tr><td colspan=2><u>" << title;
-	if(!m_bVerbose)
+	if(!m_bVerbose && hidden>0)
 		m_fmtHelper << " (Top " << sorted.size() << ", " << hidden <<  " more not displayed)";
 	m_fmtHelper << "</u></td></tr>\n";
 	for(auto it=sorted.rbegin(); it!=sorted.rend(); ++it)
@@ -2172,7 +2191,7 @@ int parseidx_demo(LPCSTR file)
 		virtual void HandlePkgEntry(const tRemoteFileInfo &entry) override
 		{
 			cout << "Dir: " << entry.sDirectory << endl << "File: " << entry.sFileName << endl
-					<< "Checksum-" << entry.fpr.GetCsName() << ": " << entry.fpr.GetCsAsString()
+					<< "Checksum-" << GetCsName(entry.fpr.csType) << ": " << entry.fpr.GetCsAsString()
 					<< endl << "ChecksumUncompressed: " << entry.bInflateForCs << endl <<endl;
 		}
 		virtual bool ProcessRegular(const mstring &, const struct stat &) override {return true;}
