@@ -23,6 +23,8 @@ using namespace std;
 // evil hack to simulate random disconnects
 //#define DISCO_FAILURE
 
+#define MAX_RETRY 11
+
 static cmstring sGenericError("567 Unknown download error occured");
 
 std::atomic_uint g_nDlCons(0);
@@ -93,6 +95,8 @@ struct tDlJob
 		STATE_FINISHJOB
 	} tDlState;
 
+	string m_extraHeaders;
+
 	tHttpUrl m_remoteUri;
 	const tHttpUrl *m_pCurBackend=nullptr;
 
@@ -137,6 +141,31 @@ struct tDlJob
 			m_pStorage->DecDlRefCount(sErrorMsg.empty() ? sGenericError : sErrorMsg);
 	}
 
+	inline void ExtractCustomHeaders(LPCSTR reqHead)
+	{
+		if(!reqHead)
+			return;
+		header h;
+		tLPS cheaders;
+		h.Load(reqHead, std::numeric_limits<int>::max(), &cheaders);
+		if(cheaders.empty())
+			return;
+		for(const auto& ch: cheaders)
+		{
+			// some headers are reserved (actually all starting with those prefixes
+			if(strncasecmp(ch.first.c_str(), WITHLEN("Host")) &&
+					strncasecmp(ch.first.c_str(), WITHLEN("Proxy-Authorization")) &&
+					strncasecmp(ch.first.c_str(), WITHLEN("Cache-Control")) &&
+					strncasecmp(ch.first.c_str(), WITHLEN("Accept")) &&
+					strncasecmp(ch.first.c_str(), WITHLEN("User-Agent"))
+					)
+			{
+				m_extraHeaders += ch.first;
+				m_extraHeaders += ch.second;
+			}
+		}
+	}
+
 	inline string RemoteUri(bool bUrlEncoded)
 	{
 		if(m_pCurBackend)
@@ -361,7 +390,9 @@ struct tDlJob
 		if (acfg::exporigin && !xff.empty())
 			head << "X-Forwarded-For: " << xff << "\r\n";
 
-		head << acfg::requestapx << "Accept: */*\r\nAccept-Encoding: identity\r\nConnection: "
+		head << acfg::requestapx
+				<< m_extraHeaders
+				<< "Accept: */*\r\nAccept-Encoding: identity\r\nConnection: "
 				<< (acfg::persistoutgoing ? "keep-alive\r\n\r\n" : "close\r\n\r\n");
 
 #ifdef SPAM
@@ -427,10 +458,10 @@ struct tDlJob
 				bool bHotItem = (m_DlState == STATE_REGETHEADER);
 				dbgline;
 
-				int l = h.LoadFromBuf(inBuf.rptr(), inBuf.size());
-				if (0 == l)
+				auto hDataLen = h.LoadFromBuf(inBuf.rptr(), inBuf.size());
+				if (0 == hDataLen)
 					return HINT_MORE;
-				if (l<0)
+				if (hDataLen<0)
 				{
 					dbgline;
 					sErrorMsg = "500 Invalid header";
@@ -438,8 +469,8 @@ struct tDlJob
 					return EFLAG_MIRROR_BROKEN | HINT_DISCON | HINT_KILL_LAST_FILE;
 				}
 
-				ldbg("contents: " << std::string(inBuf.rptr(), l));
-				inBuf.drop(l);
+				ldbg("contents: " << std::string(inBuf.rptr(), hDataLen));
+				inBuf.drop(hDataLen);
 				if (h.type != header::ANSWER)
 				{
 					dbgline;
@@ -542,7 +573,8 @@ struct tDlJob
 					DropDnsCache();
 				}
 
-				if(!m_pStorage->DownloadStartedStoreHeader(h, inBuf.rptr(), bHotItem, bDoRetry))
+				if(!m_pStorage->DownloadStartedStoreHeader(h, hDataLen,
+						inBuf.rptr(), bHotItem, bDoRetry))
 				{
 					if(bDoRetry)
 						return EFLAG_LOST_CON | HINT_DISCON; // recoverable
@@ -569,7 +601,7 @@ struct tDlJob
 			{
 				ldbg("STATE_FINISHJOB");
 				m_DlState = STATE_GETHEADER;
-				m_pStorage->StoreFileData(NULL, 0);
+				m_pStorage->StoreFileData(nullptr, 0);
 				return HINT_DONE | m_eReconnectASAP;
 			}
 			else if (m_DlState == STATE_GETCHUNKHEAD)
@@ -584,7 +616,7 @@ struct tDlJob
 					inBuf.drop(1);
 				}
 				const char *crlf(0), *pStart(inBuf.c_str());
-				if (!inBuf.size() || NULL == (crlf = strstr(pStart, "\r\n")))
+				if (!inBuf.size() || nullptr == (crlf = strstr(pStart, "\r\n")))
 				{
 					inBuf.move();
 					return HINT_MORE;
@@ -653,7 +685,7 @@ void dlcon::wake()
 
 bool dlcon::AddJob(tFileItemPtr m_pItem, const tHttpUrl *pForcedUrl,
 		const acfg::tRepoData *pBackends,
-		cmstring *sPatSuffix)
+		cmstring *sPatSuffix, LPCSTR reqHead)
 {
 	if(!pForcedUrl)
 	{
@@ -670,9 +702,12 @@ bool dlcon::AddJob(tFileItemPtr m_pItem, const tHttpUrl *pForcedUrl,
 
 	LOGSTART2("dlcon::EnqJob", todo->m_remoteUri.ToURI(false));
 */
-	m_qNewjobs.push_back(
+	m_qNewjobs.emplace_back(
 			make_shared<tDlJob>(this, m_pItem, pForcedUrl, pBackends, sPatSuffix,
 							m_bManualMode ? ACFG_REDIRMAX_DEFAULT : acfg::redirmax));
+
+	m_qNewjobs.back()->ExtractCustomHeaders(reqHead);
+
 	wake();
 	return true;
 }
@@ -757,7 +792,7 @@ inline uint dlcon::ExchangeData(mstring &sErrorMsg, tTcpHandlePtr &con, tDljQueu
 			goto proc_data;
 		}
 
-		r=select(nMaxFd + 1, &rfds, &wfds, NULL, &tv);
+		r=select(nMaxFd + 1, &rfds, &wfds, nullptr, &tv);
 		ldbg("returned: " << r << ", errno: " << errno);
 
 		if (r < 0)
@@ -896,14 +931,20 @@ inline uint dlcon::ExchangeData(mstring &sErrorMsg, tTcpHandlePtr &con, tDljQueu
 
 #ifdef DISCO_FAILURE
 #warning hej
-			static int fakeFail=3;
+			static int fakeFail=-123;
+			if(fakeFail == -123)
+			{
+				srand(getpid());
+				fakeFail = rand()%123;
+			}
 			if( fakeFail-- < 0)
 			{
 //				LOGLVL(LOG_DEBUG, "\n#################\nFAKING A FAILURE\n###########\n");
 				r=0;
-				fakeFail=10;
+				fakeFail=rand()%123;
 				errno = EROFS;
-				r = -errno;
+				//r = -errno;
+				shutdown(con.get()->GetFD(), SHUT_RDWR);
 			}
 #endif
 
@@ -911,7 +952,7 @@ inline uint dlcon::ExchangeData(mstring &sErrorMsg, tTcpHandlePtr &con, tDljQueu
 			{
 				ldbg("why EAGAIN/EINTR after getting it from select?");
 //				timespec sleeptime = { 0, 432000000 };
-//				nanosleep(&sleeptime, NULL);
+//				nanosleep(&sleeptime, nullptr);
 				goto loop_again;
 			}
 			else if (r == 0)
@@ -1042,8 +1083,7 @@ void dlcon::WorkLoop()
 
 	bool bStopRequesting=false; // hint to stop adding request headers until the connection is restarted
 
-	int nLostConTolerance=0;
-#define MAX_RETRY 5
+	int nLostConTolerance=MAX_RETRY;
 
 	auto BlacklistMirror = [&](tDlJobPtr & job)
 	{
@@ -1130,18 +1170,15 @@ void dlcon::WorkLoop()
 							sErrorMsg,
 							&bUsed,
 							m_qNewjobs.front()->GetConnStateTracker(),
-#ifdef HAVE_SSL
-						tgt.bSSL,
-#else
-						false
-#endif
-						timeout, fresh);
+							IFSSLORFALSE(tgt.bSSL),
+							timeout, fresh);
 			}	;
 
 				auto& cjob = m_qNewjobs.front();
 				auto proxy = m_bProxyTot ? nullptr : prefProxy(cjob);
 				auto& peerHost = cjob->GetPeerHost();
 
+#ifdef HAVE_SSL
 				if(peerHost.bSSL)
 				{
 					if(proxy)
@@ -1158,6 +1195,7 @@ void dlcon::WorkLoop()
 						con = doconnect(peerHost, acfg::nettimeout, false);
 				}
 				else
+#endif
 				{
 					if(proxy)
 					{
@@ -1241,7 +1279,7 @@ void dlcon::WorkLoop()
 
 				cjob->AppendRequest(m_sendBuf, m_sXForwardedFor, proxy);
 				LOG("request added to buffer");
-				inpipe.push_back(cjob);
+				inpipe.emplace_back(cjob);
 				m_qNewjobs.pop_front();
 
 				if (m_nTempPipelineDisable > 0)
@@ -1322,8 +1360,10 @@ void dlcon::WorkLoop()
 				nLostConTolerance=MAX_RETRY;
 			}
 
+			con.reset();
+
 			timespec sleeptime = { 0, 325000000 };
-			nanosleep(&sleeptime, NULL);
+			nanosleep(&sleeptime, nullptr);
 
 			// trying to resume that job secretly, unless user disabled the use of range (we
 			// cannot resync the sending position ATM, throwing errors to user for now)
