@@ -11,6 +11,7 @@
 
 #include "md5.h"
 #include "sha1.h"
+#include "sha2.h"
 #include "csmapping.h"
 #include "aclogger.h"
 #include "filelocks.h"
@@ -63,16 +64,18 @@ public:
 	}
 };
 
+#ifdef SIGBUSHUNTING
 // this weird kludge is only needed in order to access this data from POSIX signal handlers
 // which needs to be both, reliable and lock-free
 tMmapEntry g_mmapMemory[10];
 lockable g_mmapMemoryLock;
+#endif
 
 class IDecompressor
 {
 public:
 	bool eof=false;
-	mstring *psError = NULL;
+	mstring *psError = nullptr;
 	virtual ~IDecompressor() {};
 	virtual bool UncompMore(char *szInBuf, size_t nBufSize, size_t &nBufPos, acbuf &UncompBuf) =0;
 	virtual bool Init() =0;
@@ -244,11 +247,6 @@ bool filereader::OpenFile(const string & sFilename, bool bNoMagic, uint nFakeTra
 	Close(); // reset to clean state
 	m_nEofLines=nFakeTrailingNewlines;
 
-	// this makes sure not to truncate file while it's mmaped
-	m_mmapLock = filelocks::Acquire(sFilename);
-	// namedmutex mmapMx(g_noTruncateLocks, sFilename);
-	// lockguard guardWriteMx(mmapMx);
-
 	m_fd = open(sFilename.c_str(), O_RDONLY);
 #ifdef SHRINKTEST
 	m_fd = open(sFilename.c_str(), O_RDWR);
@@ -318,6 +316,9 @@ bool filereader::OpenFile(const string & sFilename, bool bNoMagic, uint nFakeTra
 		return false;
 	}
 
+	// this makes sure not to truncate file while it's mmaped
+	m_mmapLock = TFileShrinkGuard::Acquire(statbuf);
+
 	// LFS on 32bit? That's not good for mmap. Don't risk incorrect behaviour.
 	if(uint64_t(statbuf.st_size) >  MAX_VAL(size_t))
     {
@@ -344,7 +345,7 @@ bool filereader::OpenFile(const string & sFilename, bool bNoMagic, uint nFakeTra
 	}
 	else
 	{
-		m_szFileBuf = NULL;
+		m_szFileBuf = nullptr;
 		m_nBufSize = 0;
 	}
 	
@@ -367,6 +368,7 @@ bool filereader::OpenFile(const string & sFilename, bool bNoMagic, uint nFakeTra
 	m_nCurLine=0;
 	m_bError = m_bEof = false;
 
+#ifdef SIGBUSHUNTING
 	{
 		// try to keep a thread reference in some free slot
 		lockguard g(g_mmapMemoryLock);
@@ -380,6 +382,7 @@ bool filereader::OpenFile(const string & sFilename, bool bNoMagic, uint nFakeTra
 			}
 		}
 	}
+#endif
 	return true;
 }
 
@@ -404,6 +407,7 @@ void filereader::Close()
 
 	if (m_szFileBuf != MAP_FAILED)
 	{
+#ifdef SIGBUSHUNTING
 		{
 			lockguard g(g_mmapMemoryLock);
 			for (auto &x : g_mmapMemory)
@@ -414,12 +418,14 @@ void filereader::Close()
 					x.valid = false;
 			}
 		}
+#endif
 
 		munmap(m_szFileBuf, m_nBufSize);
 		m_szFileBuf = (char*) MAP_FAILED;
 	}
 
 	checkforceclose(m_fd);
+	m_mmapLock.reset();
 	m_Dec.reset();
 
 	m_nBufSize=0;
@@ -436,15 +442,25 @@ std::atomic_int g_nThreadsToKill;
 
 void handle_sigbus()
 {
-	if(!acfg::sigbuscmd.empty())
+	if (!acfg::sigbuscmd.empty())
 	{
 		/*static char buf[21];
-		sprintf(buf, "%u", (unsigned) getpid());
-		setenv("ACNGPID", buf, 1);
-		*/
+		 sprintf(buf, "%u", (unsigned) getpid());
+		 setenv("ACNGPID", buf, 1);
+		 */
 		::ignore_value(system(acfg::sigbuscmd.c_str()));
 	}
+	else
+	{
+
+		aclog::err(
+				"FATAL ERROR: apparently an IO error occurred, while reading files. "
+						"Please check your system logs for related errors reports. Also consider "
+						"using the BusAction option, see Apt-Cacher NG Manual for details");
+	}
 #if 1
+
+#ifdef SIGBUSHUNTING
 	for (auto &x : g_mmapMemory)
 	{
 		if (!x.valid.load())
@@ -452,6 +468,7 @@ void handle_sigbus()
 		aclog::err(string("FATAL ERROR: probably IO error occurred, probably while reading the file ")
 			+x.path+" . Please check your system logs for related errors.");
 	}
+#endif
 
 #else
 	// attempt to pass the signal around threads, stopping the one troublemaker
@@ -599,7 +616,7 @@ bool filereader::GetOneLine(string & sOut, bool bForceUncompress) {
 
 #ifndef MINIBUILD
 
-#ifdef HAVE_SSL
+#ifdef USE_SSL_CS
 class csumSHA1 : public csumBase, public SHA_CTX
 {
 public:
@@ -607,6 +624,20 @@ public:
 	void add(const char *data, size_t size) override { SHA1_Update(this, (const void*) data, size); }
 	void finish(uint8_t* ret) override { SHA1_Final(ret, this); }
 };
+class csumSHA256 : public csumBase, public SHA256_CTX
+{
+public:
+	csumSHA256() { SHA256_Init(this); }
+	void add(const char *data, size_t size) override { SHA256_Update(this, (const void*) data, size); }
+	void finish(uint8_t* ret) override { SHA256_Final(ret, this); }
+};
+class csumSHA512 : public csumBase, public SHA512_CTX
+{
+public:
+	csumSHA512() { SHA512_Init(this); }
+	void add(const char *data, size_t size) override { SHA512_Update(this, (const void*) data, size); }
+	void finish(uint8_t* ret) override { SHA512_Final(ret, this); }
+};
 class csumMD5 : public csumBase, public MD5_CTX
 {
 public:
@@ -622,6 +653,20 @@ public:
 	void add(const char *data, size_t size) override { sha_update(this, (SHA_BYTE*) data, size); }
 	void finish(uint8_t* ret) override { sha_final(ret, this); }
 };
+class csumSHA256 : public csumBase, public sha256_ctx
+{
+public:
+	csumSHA256() { sha256_init(this); }
+	void add(const char *data, size_t size) override { sha256_update(this, (uint8_t*) data, size); }
+	void finish(uint8_t* ret) override { sha256_final(this, ret); }
+};
+class csumSHA512 : public csumBase, public sha512_ctx
+{
+public:
+	csumSHA512() { sha512_init(this); }
+	void add(const char *data, size_t size) override { sha512_update(this, (SHA_BYTE*) data, size); }
+	void finish(uint8_t* ret) override { sha512_final(this, ret); }
+};
 class csumMD5 : public csumBase, public md5_state_s
 {
 public:
@@ -638,8 +683,13 @@ std::unique_ptr<csumBase> csumBase::GetChecker(CSTYPES type)
 	case CSTYPE_MD5:
 		return std::unique_ptr<csumBase>(new csumMD5);
 	case CSTYPE_SHA1:
-	default: // for now
 		return std::unique_ptr<csumBase>(new csumSHA1);
+	case CSTYPE_SHA256:
+		return std::unique_ptr<csumBase>(new csumSHA256);
+	case CSTYPE_SHA512:
+		return std::unique_ptr<csumBase>(new csumSHA512);
+	default: // for now
+		return std::unique_ptr<csumBase>();
 	}
 }
 
@@ -730,18 +780,18 @@ bool Bz2compressFile(const char *pathIn, const char*pathOut)
 {
 	bool bRet=false;
 	filereader reader;
-	FILE *f(NULL);
-	BZFILE *bzf(NULL);
+	FILE *f(nullptr);
+	BZFILE *bzf(nullptr);
 	int nError(0);
 
 	if(!reader.OpenFile(pathIn, true))
 		return false;
 
-	if(NULL !=(f = fopen(pathOut, "w")))
+	if(nullptr !=(f = fopen(pathOut, "w")))
 	{
 		if(!ferror(f))
 		{
-			if(NULL != (bzf = BZ2_bzWriteOpen( &nError, f, 9, 0, 30)))
+			if(nullptr != (bzf = BZ2_bzWriteOpen( &nError, f, 9, 0, 30)))
 			{
 				if(BZ_OK == nError)
 				{
