@@ -60,7 +60,7 @@ protected:
 
 public:
 	tPassThroughFitem(std::string s) :
-	m_pData(NULL), m_nConsumable(0), m_nConsumed(0)
+	m_pData(nullptr), m_nConsumable(0), m_nConsumed(0)
 	{
 		m_sPathRel = s;
 		m_bAllowStoreData=false;
@@ -73,7 +73,7 @@ public:
 	}
 	virtual int GetFileFd() override
 			{ return SPECIAL_FD; }; // something, don't care for now
-	virtual bool DownloadStartedStoreHeader(const header & h, const char *,
+	virtual bool DownloadStartedStoreHeader(const header & h, size_t, const char *,
 			bool, bool&) override
 	{
 		setLockGuard;
@@ -116,7 +116,7 @@ public:
 
 			dbgline;
 			m_nConsumable=0;
-			m_pData=NULL;
+			m_pData=nullptr;
 			return m_nConsumed;
 		}
 		return true;
@@ -190,57 +190,13 @@ public:
 		m_head.set(header::CONTENT_LENGTH, m_nSizeChecked);
 	}
 	// never used to store data
-	bool DownloadStartedStoreHeader(const header &, const char *, bool, bool&)
+	bool DownloadStartedStoreHeader(const header &, size_t, const char *, bool, bool&)
 	override
 	{return false;};
 	bool StoreFileData(const char *, unsigned int) override {return false;};
 	header & HeadRef() { return m_head; }
 };
 
-ssize_t sendfile_generic(int out_fd, int in_fd, off_t *offset, size_t count)
-{
-	char buf[8192];
-	ssize_t totalcnt=0;
-	
-	if(!offset)
-	{
-		errno=EFAULT;
-		return -1;
-	}
-	if(lseek(in_fd, *offset, SEEK_SET)== (off_t)-1)
-		return -1;
-	while(count>0)
-	{
-		auto readcount=read(in_fd, buf, min(count, sizeof(buf)));
-		if(readcount<=0)
-		{
-			if(errno==EINTR || errno==EAGAIN)
-				continue;
-			else
-				return readcount;
-		}
-		
-		*offset+=readcount;
-		totalcnt+=readcount;
-		count-=readcount;
-		
-		for(decltype(readcount) nPos(0);nPos<readcount;)
-		{
-			auto r=write(out_fd, buf+nPos, readcount-nPos);
-			if(r==0) continue; // not nice but needs to deliver it
-			if(r<0)
-			{
-				if(errno==EAGAIN || errno==EINTR)
-					continue;
-				return r;
-			}
-			nPos+=r;
-		}
-	}
-	return totalcnt;
-}
-
-
 job::job(header *h, con *pParent) :
 	m_filefd(-1),
 	m_pParentCon(pParent),
@@ -493,7 +449,7 @@ inline bool job::ParseRange()
 	return false;
 }
 
-void job::PrepareDownload() {
+void job::PrepareDownload(LPCSTR headBuf) {
 
     LOGSTART("job::PrepareDownload");
     
@@ -707,7 +663,8 @@ MYTRY
 
 					if (m_pParentCon->m_pDlClient->AddJob(m_pItem.get(),
 							bHaveRedirects ? nullptr : &theUrl, repoMapping.repodata,
-							bHaveRedirects ? &repoMapping.sRestPath : nullptr))
+							bHaveRedirects ? &repoMapping.sRestPath : nullptr,
+									(LPCSTR) ( bPtMode ? headBuf : nullptr)))
 				{
 					ldbg("Download job enqueued for " << m_sFileLoc);
 				}
@@ -732,7 +689,7 @@ report_overload:
 
 report_notallowed:
 	SetErrorResponse((tSS() << "403 Forbidden file type or location: " << sReqPath).c_str(),
-			NULL, "403 Forbidden file type or location");
+			nullptr, "403 Forbidden file type or location");
 //    USRDBG( sRawUriPath + " -- ACCESS FORBIDDEN");
     return ;
 
@@ -848,9 +805,6 @@ job::eJobResult job::SendData(int confd)
 					const char *szErr = BuildAndEnqueHeader(fistate, nGoodDataSize, respHead);
 					if(szErr) THROW_ERROR(szErr);
 					USRDBG("Response header to be sent in the next cycle: \n" << m_sendbuf );
-#ifdef DEBUG
-#warning TODO: review the life cycle, might avoid one select call in the function above (prio:low)
-#endif
 					return R_AGAIN;
 				}
 				case(STATE_HEADER_SENT):
@@ -972,8 +926,10 @@ job::eJobResult job::SendData(int confd)
 				
 				case(STATE_ALLDONE):
 					LOG("State: STATE_ALLDONE?");
+				// no break
 				case (STATE_ERRORCONT):
 					LOG("or STATE_ERRORCONT?");
+				// no break
 				case(STATE_FINISHJOB):
 					LOG("or STATE_FINISHJOB");
 					{
@@ -984,6 +940,7 @@ job::eJobResult job::SendData(int confd)
 					}
 					break;
 				case(STATE_TODISCON):
+						// no break
 				default:
 					return R_DISCON;
 			}
